package eu.mhsl.craftattack.worldmuseum;

import eu.mhsl.craftattack.worldmuseum.commands.GamemodeCommand;
import eu.mhsl.craftattack.worldmuseum.commands.GcCommand;
import eu.mhsl.craftattack.worldmuseum.commands.TeleportCommand;
import eu.mhsl.craftattack.worldmuseum.listener.*;
import eu.mhsl.craftattack.worldmuseum.skins.SkinCache;
import eu.mhsl.craftattack.worldmuseum.util.Config;
import eu.mhsl.craftattack.worldmuseum.worlds.World;
import eu.mhsl.craftattack.worldmuseum.worlds.WorldManager;
import eu.mhsl.craftattack.worldmuseum.util.CompassManager;
import net.minestom.server.MinecraftServer;
import net.minestom.server.entity.GameMode;
import net.minestom.server.entity.Player;
import net.minestom.server.entity.PlayerSkin;
import net.minestom.server.event.GlobalEventHandler;
import net.minestom.server.event.inventory.InventoryPreClickEvent;
import net.minestom.server.event.player.*;
import net.minestom.server.instance.InstanceManager;
import net.minestom.server.timer.TaskSchedule;

public class Main {
    static int PORT = 25565;
    static String IP = "0.0.0.0";

    public static void main(String[] args) {
        // Initialization
        System.setProperty("minestom.chunk-view-distance", "16");
        MinecraftServer minecraftServer = MinecraftServer.init();
        MinecraftServer.setChunkViewDistance(8);

        //temp
        MinecraftServer.getSchedulerManager().scheduleTask(new TablistUpdateTask(), TaskSchedule.tick(20), TaskSchedule.tick(20));

        //load eu.mhsl.craftattack.worldmuseum.worlds / configExample file
        WorldManager worldManager = WorldManager.getInstance();
        worldManager.loadWorlds();

        //load main config
        Config config = Config.getInstance();
        config.loadConfig();
        World startworld = config.getStart_world();

        //CompassManager
        CompassManager compassManager = CompassManager.getInstance();
        compassManager.loadCommpassManager();

        // Add an event callback to specify the spawning instance (and the spawn position)
        GlobalEventHandler globalEventHandler = MinecraftServer.getGlobalEventHandler();

        //listeners
        globalEventHandler.addListener(PlayerUseItemEvent.class, new CompassUseListener());
        globalEventHandler.addListener(PlayerChunkUnloadEvent.class, new ChunkUnloading());
        globalEventHandler.addListener(InventoryPreClickEvent.class, new InventoryClickListener());
        globalEventHandler.addListener(PlayerBlockBreakEvent.class, new BlockBreakListener());
        globalEventHandler.addListener(PlayerBlockPlaceEvent.class, new BlockPlaceListener());

        //commands
        MinecraftServer.getCommandManager().register(new TeleportCommand());
        MinecraftServer.getCommandManager().register(new GcCommand());
        MinecraftServer.getCommandManager().register(new GamemodeCommand());

        globalEventHandler.addListener(PlayerLoginEvent.class, event -> {
            final Player player = event.getPlayer();
            SkinCache.setSkin(player);
            event.setSpawningInstance(startworld);

            player.setRespawnPoint(startworld.getSpawn());
            player.setGameMode(GameMode.CREATIVE);
            player.getInventory().setItemStack(0, compassManager.getCompass());
        });




        try {
            PORT = Integer.parseInt(args[0]);
        } catch (Exception e) {
            if (args.length != 0) System.out.println("Given port doesn't work.");
        }
        // Start the server on port default port 25565 if none is given
        System.out.println("Running on " + IP + ":" + PORT);
        minecraftServer.start(IP, PORT);
    }
}